// Unreal Fighting Framework by Kevin Poretti

#pragma once

// FF includes
#include "Input/FFInputBufferComponent.h"

// UE includes
#include "CoreMinimal.h"
#include "UObject/Interface.h"

#include "IFFStateOwnerInterface.generated.h"

UINTERFACE(MinimalAPI, NotBlueprintable)
class UFFStateOwnerInterface : public UInterface
{
	GENERATED_BODY()
};

/**
 * TODO: document
 */

// TODO: why is this an interface again?
class UNREALFIGHTINGFRAMEWORK_API IFFStateOwnerInterface
{
	GENERATED_BODY()

public:
    /**
     *	@brief Checks to see if the provided input sequence has a valid matching series of inputs in
     *	the input buffer
     *
     *	@param InInputSequence sequence of inputs to check in the input buffer
     *	@return true if the input sequence has a valid matching series of inputs in the input buffer
     */
    UFUNCTION(BlueprintCallable, Category="UFF State Owner Interface")
    virtual bool CheckInputSequence(const FFFInputSequence& InInputSequence) = 0;
	
	/**
	 *	@brief Checks to see if at least one of the provided input sequences has a valid matching
	 *	series of inputs in the input buffer
	 *
	 *	@param InInputSequences array of sequences of inputs to check in the input buffer
	 *	@return true if at least one input sequence has a valid matching series of inputs in the
	 *	input buffer
	 */
    UFUNCTION(BlueprintCallable, Category="UFF State Owner Interface")
    virtual bool CheckInputSequences(const TArray<FFFInputSequence>& InInputSequences) = 0;

    /**
     *	@brief Convert 2D movement input a 3D world direction relative to the view orientation
     *
     *	@return direction of player's 2D movement input in world space 
     */
    virtual FVector GetMoveInputAsWorldDirection() const = 0;

    /**
     *	@brief Sets the current tick's input in the input buffer as disabled, preventing it from
     *	being considered as valid input for input sequence interpretation
     */
    virtual void DisableMostRecentInput() = 0;
};
